use strict;
use warnings;

use Test::NoWarnings;
use Test::Pod::Coverage 'tests' => 2;

# Test.
pod_coverage_ok('Test::Shared::Fixture::Wikibase::Datatype::Item::Wikidata::Dog', 'Test::Shared::Fixture::Wikibase::Datatype::Item::Wikidata::Dog is covered.');
